// Copyright (C) 2018  Nexedi SA and Contributors.
//                     Kirill Smelkov <kirr@nexedi.com>
//
// This program is free software: you can Use, Study, Modify and Redistribute
// it under the terms of the GNU General Public License version 3, or (at your
// option) any later version, as published by the Free Software Foundation.
//
// You can also Link and Combine this program with other software covered by
// the terms of any of the Free Software licenses or any of the Open Source
// Initiative approved licenses and Convey the resulting work. Corresponding
// source of such a combination shall include the source code for all other
// software used.
//
// This program is distributed WITHOUT ANY WARRANTY; without even the implied
// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// See COPYING file for full licensing terms.
// See https://www.nexedi.com/licensing for rationale and options.

package main
// ZBlk* + ZBigFile loading


import (
	//"context"

	"lab.nexedi.com/kirr/neo/go/zodb"
	"lab.nexedi.com/kirr/neo/go/zodb/btree"
	pickle "github.com/kisielk/og-rek"
)

// module of Wendelin ZODB py objects
const zwendelin = "wendelin.bigfile.file_zodb"

// ---- ZBlk0 ----

// ZBlk0 mimics ZBlk0 from python.
type ZBlk0 struct {
	*zodb.PyPersistent

	blkdata string
}

func (zb *ZBlk0) DropState() {
	zb.blkdata = ""
}

func (zb *ZBlk0) PySetState(pystate interface{}) error {
	blkdata, ok := pystate.(string)
	if !ok {
		// XXX
	}

	zb.blkdata = blkdata
	return nil
}

// ---- ZBlk1 ---

// ZData mimics ZData from python.
type ZData struct {
	*zodb.PyPersistent

	data string
}

func (zd *ZData) DropState() {
	zd.data = ""
}

func (zd *ZData) PySetState(pystate interface{}) error {
	data, ok := pystate.(string)
	if !ok {
		// XXX
	}

	zd.data = data
	return nil
}

// ZBlk1 mimics ZBlk1 from python.
type ZBlk1 struct {
	*zodb.PyPersistent

	chunktab *btree.BTree // {} offset -> ZData(chunk)
}

func (zb *ZBlk1) DropState() {
	zb.chunktab = nil
}

func (zb *ZBlk1) PySetState(pystate interface{}) error {
	chunktab, ok := pystate.(*btree.BTree)
	if !ok {
		// XXX
	}

	zb.chunktab = chunktab
	return nil
}


// ----------------------------------------

// ZBigFile mimics ZBigFile from python.
type ZBigFile struct {
	*zodb.PyPersistent

	blksize	int64
	blktab  *btree.BTree	// LOBtree{}  blk -> ZBlk*(blkdata)
}


// DropState implements Stateful.
func (bf *ZBigFile) DropState() {
	bf.blksize = -1
	bf.blktab  = nil
}

// PySetState implements PyStateful.
func (bf *ZBigFile) PySetState(pystate interface{}) (err error) {
	// ZBigFile
	//	.blksize	xint
	//	.blktab		LOBtree{}  blk -> ZBlk*(blkdata)
	//
	// state: (.blksize, .blktab)

	t, ok := pystate.(pickle.Tuple)
	if !ok || len(t) != 2 {
		// XXX
	}

	// blksize, ok := pickletools.Xint64(t[0])	// XXX reenable (pickletools)
	if !ok {
		// XXX
	}

	blktab, ok := t[1].(*btree.BTree)
	if !ok {
		// XXX
	}

	bf.blksize = 0		// XXX blksize
	bf.blktab  = blktab
	return nil
}


// ----------------------------------------

func zblk0New(base *zodb.PyPersistent) zodb.IPyPersistent	{ return &ZBlk0{PyPersistent: base}		}
func zblk1New(base *zodb.PyPersistent) zodb.IPyPersistent	{ return &ZBlk1{PyPersistent: base}		}
func zdataNew(base *zodb.PyPersistent) zodb.IPyPersistent	{ return &ZData{PyPersistent: base}		}
func zbigfileNew(pyobj *zodb.PyPersistent) zodb.IPyPersistent	{ return &ZBigFile{PyPersistent: pyobj}	}

func init() {
	zodb.PyRegisterClass(zwendelin + ".ZBlk",     zblk0New)
	zodb.PyRegisterClass(zwendelin + ".ZBlk0",    zblk0New)
	zodb.PyRegisterClass(zwendelin + ".ZBlk1",    zblk1New)
	zodb.PyRegisterClass(zwendelin + ".ZData",    zdataNew)
	zodb.PyRegisterClass(zwendelin + ".ZBigFile", zbigfileNew)
}
